#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>

gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data);
void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data);

gboolean thetimer(gpointer  data);

#ifdef __linux__
int interfaceExists(char *interface);
#endif

void updatevalue();

gboolean flash(GtkWidget  *drawingarea);

void getvalue(char *interface, unsigned long *inb, unsigned long *outb);

void bits2bytes(float f, char *s);

void timer_init(GtkWidget *widget);
