/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>

#include "bungmeter.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"


int main (int argc, char *argv[]) {
	GtkWidget *window1;

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	if(!argv[1] || strlen(argv[1]) > 6) {
		printf("Usage:  %s <network device>\n", argv[0]);
		return 1;
	}
	strncpy(interface, argv[1], 6);

	window1 = create_window1();
	gtk_widget_show(window1);
	gtk_main();

	return 0;
}
